
/****** Object:  UserDefinedFunction [dbo].[NestedGroupMembersByAccount]    Script Date: 8/17/2016 10:15:36 AM ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER OFF
GO


CREATE FUNCTION [dbo].[NestedGroupMembersByAccountNew] (@Domain  NVARCHAR(60), @Account NVARCHAR(184))
RETURNS @ReturnTable TABLE   
(AccountID BIGINT, AccountName NVARCHAR(1000))      
AS BEGIN
	-- ---------------------------------------------------------------------------------
    -- This function will take a domain name and sam account name and return the account 
	-- and all groups it is a member of.  An attempt should be made to not return groups
	-- which are deleted by joining to the domain group and computer group tables, but 
	-- there is a chance that the group is no there.
    -- ---------------------------------------------------------------------------------
    -- Declare Variables    
    -- ---------------------------------------------------------------------------------
	WITH directmembers (
		   userorgroupaccountid, 
		   accounttype,
           userorgroupdomainname,
           userorgroupaccountname, 
		   rootaccountid, 
		   rootaccount, 
		   nestedgroup,
           groupmemberaccountid, 
		   groupmemberaccount, 
		   membershippath, 
		   depth)
	AS 
	(SELECT A.AccountID,
			A.AccountType,
			A.SAMAccountDomain,
			A.SAMAccountName,
			A.AccountID,
			A.SAMAccountDomain + '\' + A.SAMAccountName,
			/* if no members, use the other version */
			Cast(A.SAMAccountDomain + '\' + A.SAMAccountName AS NVARCHAR(1000)),
			COALESCE(GMA.AccountID, A.AccountID),
			COALESCE(GMA.SAMAccountDomain + '\' + GMA.SAMAccountName, A.SAMAccountDomain + '\' + A.SAMAccountName),
			COALESCE(Cast(A.SAMAccountDomain + '\' + A.SAMAccountName + ' > ' + GMA.SAMAccountDomain + '\' + GMA.SAMAccountName AS NVARCHAR(2000)),
			Cast(A.SAMAccountDomain + '\' + A.SAMAccountName AS NVARCHAR(2000))) MembershipPath,
			CASE
			WHEN GMA.SAMAccountDomain IS NULL THEN 1
			ELSE 2
			END
	FROM   dbo.tblAccount A
			LEFT JOIN dbo.tblGroup G ON ( G.GroupAccountID = A.AccountID ) AND ( A.LDM_TOMBSTONED = 0 OR A.LDM_TOMBSTONED IS NULL )
			LEFT JOIN dbo.tblGroupMember GM ON ( G.GroupID = GM.GroupID ) AND ( GM.LDM_TOMBSTONED = 0 OR GM.LDM_TOMBSTONED IS NULL )
			LEFT JOIN dbo.tblAccount GMA ON ( GM.MemberAccountID = GMA.AccountID ) AND ( GMA.LDM_TOMBSTONED = 0 OR GMA.LDM_TOMBSTONED IS NULL )
			LEFT JOIN dbo.tblDomainUser du ON du.AccountID = GMA.AccountID
			LEFT JOIN dbo.tblDomainGroup dg ON dg.AccountID = GMA.AccountID
			LEFT JOIN dbo.tblComputerAccount cg ON cg.AccountID = GMA.AccountID
	WHERE  ( ISNULL(GMA.SAMAccountName, '') = @Account
			AND	
			(
				(ISNULL(@Domain, '') = ISNULL(GMA.SAMAccountDomain, ''))
			)
			)
			AND ( dg.LDM_TOMBSTONED = 0 OR dg.LDM_TOMBSTONED IS NULL )
			AND ( du.LDM_TOMBSTONED = 0 OR du.LDM_TOMBSTONED IS NULL )
			AND ( cg.LDM_TOMBSTONED = 0 OR cg.LDM_TOMBSTONED IS NULL )
	UNION ALL
	SELECT 
		A.AccountID,
		A.AccountType,
		A.SAMAccountDomain,
		A.SAMAccountName,
		directmembers.rootaccountid,
		directmembers.rootaccount,
		CAST(A.SAMAccountDomain + '\' + A.SAMAccountName AS NVARCHAR(1000)),
		GMA.AccountID,
		GMA.SAMAccountDomain + '\' + GMA.SAMAccountName,
		CAST(A.SAMAccountDomain + '\' + A.SAMAccountName + ' > ' + directmembers.membershippath AS NVARCHAR(2000))
		MembershipPath,
		directmembers.depth + 1
	FROM  
		directmembers
		INNER JOIN dbo.tblGroupMember GM
			ON ( directmembers.userorgroupaccountid = GM.MemberAccountID ) AND ( GM.LDM_TOMBSTONED = 0 OR GM.LDM_TOMBSTONED IS NULL )
		INNER JOIN dbo.tblGroup G 
			ON GM.GroupID = G.GroupID
		INNER JOIN dbo.tblAccount GMA 
			ON ( GMA.AccountID = GM.MemberAccountID ) AND ( GM.LDM_TOMBSTONED = 0 OR GM.LDM_TOMBSTONED IS NULL )
		INNER JOIN dbo.tblAccount A 
			ON ( A.AccountID = GM.GroupID )
	WHERE 
		directmembers.depth < 15
		AND 
			(directmembers.membershippath NOT LIKE ( '%' + CAST(a.SAMAccountDomain +'\' + ISNULL( A.SAMAccountName, '') + ' > ' AS NVARCHAR(2000)) + '%' ) )
		AND 
			( GM.LDM_TOMBSTONED = 0 )
	)
	INSERT @ReturnTable
	SELECT DISTINCT 
		directmembers.userorgroupaccountid, 
		directmembers.userorgroupdomainname + '\' + userorgroupaccountname
	FROM 
		directmembers
	LEFT JOIN dbo.tblDomainGroup dg ON dg.AccountID = directmembers.groupmemberaccountid
	LEFT JOIN dbo.tblComputerAccount cg ON cg.AccountID = directmembers.groupmemberaccountid
	WHERE 
		(dg.LDM_TOMBSTONED = 0 OR dg.LDM_TOMBSTONED IS NULL) 
		AND
		(cg.LDM_TOMBSTONED = 0 OR cg.LDM_TOMBSTONED IS NULL)
	UNION
	SELECT 
		A.AccountID,
		CASE
		WHEN ISNULL(A.SAMAccountDomain, '') = ''
		THEN A.SAMAccountName
		ELSE A.SAMAccountDomain + '\' + A.SAMAccountName
		END	
	FROM 
		dbo.tblAccount A
	WHERE 
		( A.LDM_TOMBSTONED = 0 OR A.LDM_TOMBSTONED IS NULL )
		 AND 
		( 
			ISNULL(A.SAMAccountName, '') = @Account 
			AND	
			(
				(ISNULL(@Domain, '') = ISNULL(A.SAMAccountDomain, ''))
			)
		)


      RETURN
END

GO


